@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2007 Internet2
+ *  Copyright 2001-2009 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,8 +35,8 @@ namespace opensaml {
         class SAML_API SAML1MessageDecoder : public MessageDecoder
         {
         protected:
-            SAML1MessageDecoder() {}
-            virtual ~SAML1MessageDecoder() {}
+            SAML1MessageDecoder();
+            virtual ~SAML1MessageDecoder();
 
         public:
             void extractMessageDetails (
