@@ -99,7 +99,7 @@ public:
     }
 
     void testChildElementsMarshall() {
-        QName qext("http://www.opensaml.org/", "Foo", "ext");
+        xmltooling::QName qext("http://www.opensaml.org/", "Foo", "ext");
         Conditions* conditions=ConditionsBuilder::buildConditions();
 
         //Test storing children as their direct type
